/*
mtmim.dll for MetaTrader

Copyright  2010 Erich Pribitzer

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
CmdLine plugin for Miranda IM

Copyright  2007 Cristian Libotean

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "common.h"

char retString[MAX_SIZE] = {0};
int retStrLen=0;



void PrintUsage()
{

	_lpprintf(Translate("usage: %s function_call(\"command\",\"arg1|arg2|arg3|...\",\"|\")  This will tell Miranda to run the specified command. The commands can have zero, one or more parameters. Use help command to get a list of possible commands. No command can have more than %d parameters"), DLLNAME,MAX_ARGUMENTS-1);
}

void ShowVersion()
{
	_lpprintf("%s %s", DLLNAME,DLLVERSION);
}


int dllmain(int argc, char *argv[], int timeout)
{
	int error = 0;
	if ((argc == 2) && (strcmp(argv[1], "-v") == 0))
	{
		ShowVersion();
		
		return 0;
	}
	
	if ((InitClient()) || (ConnectToMiranda()) || (GetKnownCommands()) || (LoadLangPackModule(sdCmdLine->mimFolder)))
	{
		LoadLangPackModule(".");
		_lpprintf(Translate("Could not create connection with Miranda or could not retrieve list of known commands"));
		error = MIMRES_NOMIRANDA;
	}
	else{
		if ((argc <= 1) || (argc > MAX_ARGUMENTS))
		{
			PrintUsage();
		}
		else{
			PReply reply = ParseCommand(argv, argc, timeout);
			if (reply)
			{
				error = reply->code;
				_lpprintf("%s", reply->message);
			}
			else{
				_lpprintf(Translate("Unknown command '%s'"), argv[1]);
			}
			
			DestroyKnownCommands();
			DisconnectFromMiranda();
			DestroyClient();
			LangPackShutdown();
		}
	}

	return error;
}

int _lpprintf(const char *format, ...)
{
	va_list va;
	va_start(va, format);

	retString[0] = '\0';
	int len = _vsnprintf(retString, MAX_SIZE - 1, format, va);
	retString[MAX_SIZE - 1] = 0;
	va_end(va);

	CharToOemBuff(retString, retString, len);		
	retStrLen=len;
	return len;
}

MT4_API int __stdcall m_sendCommandWaitResponse(const char * command, const char * args, const char * delimiter, int timeout)
{
	int error=0;
	retStrLen=0;
	char *argv[MAX_ARGUMENTS]={0};
	int argc=0;
	char *_args="";
	char *token=NULL;
	char *next_token=NULL;

	_args=_strdup(args);

	token = strtok_s(_args,delimiter,&next_token);

	
	if(strlen(command)>0)
	{
		argv[argc++]=_strdup("mtmim.dll");
		argv[argc++]=_strdup(command);
	}

	while (token != NULL && argc < MAX_ARGUMENTS)
	{
		argv[argc]=_strdup(token);
		argc++;
		token = strtok_s(NULL,delimiter,&next_token);		
	}

	error=dllmain(argc, argv, timeout);

	for(int t=0;t<argc;t++)
	{
		free(argv[t]);
	}
	
	free(_args);

	return error;
}

MT4_API char * m_getLastResponseString()
{
	if(retStrLen > 0)
		return retString;
	else
		return "";

}